package ewa;

import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import javax.servlet.RequestDispatcher;

import beans.*;
import java.util.*;

public class Memory extends HttpServlet
{
  protected void doGet(HttpServletRequest request, HttpServletResponse response)
    throws ServletException, IOException
  {
    response.setContentType("text/html; charset=UTF-8");

    HttpSession session = request.getSession(true);
    MemorySession memSession = (MemorySession)session.getAttribute("memSession");

    if (memSession == null || request.getParameter("newgame") != null)
    {
      memSession = new MemorySession(16, request.getRequestURI());
      dealCards(memSession, "img/card_images");
    }

    handleCardClick(memSession, request.getParameter("cardid"));

    session.setAttribute("memSession", memSession);

    RequestDispatcher dispatcher = getServletContext().getRequestDispatcher("/table.jsp");
    dispatcher.forward(request, response);
  }

  /*--------------------------------------------------------------------------*/

  public String getServletInfo()
  {
    return "EWA Memory Servlet";
  }

  /*--------------------------------------------------------------------------*/

  private void dealCards(MemorySession memSession, String path)
    throws ServletException
  {
    /* TODO: images should be read from filessystem (doesn't work with .war)
     *       or some database
     */
    String images[] = { "card1.jpg", "card2.jpg", "card3.jpg", "card4.jpg",
      "card5.jpg", "card6.jpg", "card7.jpg", "card8.jpg" };
    if (images == null)
      throw new ServletException("Unable to read card images");
    if (images.length * 2 < memSession.getTotalCards())
      throw new ServletException("Not enough cards available");

    List<String> imagelist = new ArrayList<String>();
    for (int i = 0; i < images.length; ++i)
      imagelist.add(images[i]);

    Collections.shuffle(imagelist);

    for (int i = imagelist.size(); i > memSession.getTotalCards(); --i)
      imagelist.remove(i - 1);

    imagelist.addAll(imagelist);

    Collections.shuffle(imagelist);

    for (int i = 0; i < imagelist.size(); ++i)
      memSession.getCard(i).setImageUrl(path + "/" + imagelist.get(i));
  }

  /*--------------------------------------------------------------------------*/

  private void handleCardClick(MemorySession memSession, String idparam)
  {
    if (idparam == null)
      return;

    int cardid = 0;
    try
    {
      cardid = Integer.parseInt(idparam);
    }
    catch(NumberFormatException ex)
    {
      return;
    }

    if (cardid < 0 || cardid >= memSession.getTotalCards())
      return;

    handleCardClick(memSession, memSession.getCard(cardid));
  }

  /*--------------------------------------------------------------------------*/

  private void handleCardClick(MemorySession memSession, MemoryCard card)
  {
    if (card.isFound())
      return;

    if (memSession.getFirstCard() == null)
    {
      /* erste karte wird aufgedeckt. decke alle anderen karten zu */
      for (int i = 0; i < memSession.getTotalCards(); ++i)
        memSession.getCard(i).setFlipped(false);
      card.setFlipped(true);
      memSession.setFirstCard(card);
    }
    else if (memSession.getFirstCard() == card)
    {
      /* karte wieder zudecken  */
      card.setFlipped(false);
      memSession.incTrys();
      memSession.setFirstCard(null);
    }
    else
    {
      /* zweite karte wird aufgedeckt */
      if (memSession.getFirstCard().getImageUrl().equals(card.getImageUrl()))
      {
        /* paar wurde gefunden */
        memSession.getFirstCard().setFound(true);
        card.setFound(true);
        memSession.incFoundPairs();
      }
      else
      {
        /* paar wurde nicht gefunden */
        card.setFlipped(true);
        memSession.incTrys();
      }
      memSession.setFirstCard(null);
    }
  }
}
